function parseDate( dateStr ) {
	var year = 1970;
	var month = 1;
	var day = 1;
	var hour = 0;
	var minute = 0;
	var sec = 0;
	if ( dateStr.length > 3 ) {
		year = parseInt( dateStr.substring( 0, 4 ), 10 );
		if ( dateStr.length > 5 ) {
			month = parseInt( dateStr.substring( 4, 6 ), 10 );
			if ( dateStr.length > 7 ) {
				day = parseInt( dateStr.substring( 6, 8 ), 10 );
				if ( dateStr.length > 9 ) {
					var pos = 8;
					if ( dateStr.charAt( 8 ) < '0' || dateStr.charAt( 8 ) > '9' ) {
						pos++;
					}
					if ( dateStr.length > pos + 1 ) {
						hour = parseInt( dateStr.substring( pos, pos + 2 ), 10 );
						pos += 2;
						if ( dateStr.length > pos + 1 ) {
							minute = parseInt( dateStr.substring( pos, pos + 2 ), 10 );
							pos += 2;
							if ( dateStr.length > pos + 1 ) {
								sec = parseInt( dateStr.substring( pos, pos + 2 ), 10 );
							}
						}
					}
				}
			}
		}
	}
	return new Date( year, month - 1, day, hour, minute, sec );
};

function handleTLCMouseClick( e, id ) {
	var tlc = ImtlcMap[ id ];
	if ( !e ) {
		e = window.event;
	}
	if ( e.ctrlKey ) {
		tlc.setSelectedState( !tlc.isSelected && tlc.isOpenable );
	}
	else {
		tlc.parent.setActual( id );
	}
	// alert( tlc.ID + ' -- ' + e );
};

var ImtlcMap = new Array();

/**
 * @param id
 * @param date
 * @param content
 * @returns {IMTimelineCell}
 */
function IMTimelineCell( props ) { // id, styiuid, date, content, href
	for ( var prop in props ) {
		this[ prop ] = props[ prop ];
	}
	var id = this.ID;
	ImtlcMap[ id ] = this;
	this.div = document.createElement( 'div' );
	this.div.id = "TL_DIV_" + id;
	var icon = document.createElement( 'div' );
	icon.innerHTML = "&nbsp;";
	icon.className = "status";
	this.div.appendChild( icon );

	if ( this.href ) {
		var href = this.href;
		this.div.ondragstart = function( e ) {
			startDrag( e, href );
		};
		var link = document.createElement( 'a' );
		link.innerHTML = this.content;
		link.onclick = function( e ) {
			handleTLCMouseClick( e, id );
			return false;
		};
		if ( isIE ) { // isIE comes from LayoutScripts.js
			link.href = "#";
			link.draggable = true;
			link.ondragstart = function( e ) {
				startDrag( e, href );
			};
		}
		else {
			link.href = this.href;
		}
		this.div.appendChild( link );
	}
	else {
		var span = document.createElement( 'span' );
		span.innerHTML = this.content;
		this.div.appendChild( span );
	}

	// add indicator icons
	var iconClasses = [ "foldicon", "icon RelRelIcon", "icon RelBodyIcon", "icon RelModyIcon", "icon KeyImageIcon", "icon AttachmentIcon", "icon NoteIcon" ];
	for ( var i = 0; i < iconClasses.length; i++ ) {
		icon = document.createElement( 'div' );
		icon.innerHTML = "&nbsp;";
		icon.className = iconClasses[ i ];
		this.div.appendChild( icon );
	}
	this.div.title = htmlDecode( this.content );

	this.div.onclick = function( e ) {
		handleTLCMouseClick( e, id );
	};

	if ( typeof ( this.date ) == 'string' ) {
		this.date = parseDate( this.date );
	}
	else if ( this.date == null ) {
		this.date = parseDate( "" );
	}
	else if ( !( this.date instanceof Date ) ) {
		throw "bad date parameter for IMTimelineCell: " + this.date;
	}

	this.setParent = function( tl ) {
		this.parent = tl;
	};

	this.checkOverflow = function() {
		var isOverflowing = this.div.clientWidth < this.div.scrollWidth || this.div.clientHeight < this.div.scrollHeight;
		return isOverflowing;
	};

	this.setSelectedState = function( selected ) {
		if ( this.isSelected == selected ) {
			return;
		}
		this.isSelected = selected;
		this.setClass();
	};

	this.setRequestedState = function( requested ) {
		if ( this.isRequested == requested ) {
			return;
		}
		this.isRequested = requested;
		this.setClass();
	};

	this.setActualState = function( actual ) {
		if ( this.isActual == actual ) {
			return;
		}
		this.isActual = actual;
		if ( actual ) {
			// we may want to do something... like initialize an iframe...
			if ( this.parent.iframeID != null ) {
				loadImFrame( this.parent.iframeID, id );
			}
		}
		this.setClass();
	};

	this.setType = function( type ) {
		if ( this.type == type ) {
			return;
		}
		this.type = type;
		this.setClass();
	};

	this.setNoteState = function( hasNote ) {
		if ( this.hasNote == hasNote ) {
			return;
		}
		this.hasNote = hasNote;
		this.setClass();
	};

	this.setClass = function() {
		var classNames = "TLDivCommon TLDivType" + this.type;
		if ( this.hasNote ) {
			classNames += " TLDivNote";
		}
		if ( this.hasAttachment ) {
			classNames += " TLDivAttachment";
		}
		if ( this.hasKeyImages ) {
			classNames += " TLDivKeyImage";
		}
		if ( this.isSameModality ) {
			classNames += " TLDivRelMody";
		}
		if ( this.isSameBodypart ) {
			classNames += " TLDivRelBody";
		}
		if ( this.isRelevant ) {
			classNames += " TLDivRelRel";
		}
		if ( this.isBase ) {
			classNames += " TLDivBase";
		}
		if ( this.isActual ) {
			classNames += " TLDivActual";
		}
		if ( this.isFinal ) {
			classNames += " TLDivFinal";
		}

		if ( ( this.type == "Study" || this.type == "Remote" ) && this.isRequested ) {
			classNames += " TLDivRequested";
		}
		this.div.className = classNames;
	};

	this.setClass();
};

Timeline.prototype.setActual = function( ID ) {
	for ( var i = 0; i < this.objects.length; i++ ) {
		this.objects[ i ].setActualState( this.objects[ i ].ID == ID );
	}
	if ( typeof this.tlbList != "undefined" ) {
		for ( var j = 0; j < this.tlbList.length; j++ ) {
			this.tlbList[ j ].refresh();
		}
	}
};

Timeline.prototype.setNoteState = function( ID, hasNote ) {
	for ( var i = 0; i < this.objects.length; i++ ) {
		if ( this.objects[ i ].ID == ID ) {
			this.objects[ i ].setNoteState( hasNote );
		}
	}
};

Timeline.prototype.getActualCell = function() {
	for ( var i = 0; i < this.objects.length; i++ ) {
		if ( this.objects[ i ].isActual ) {
			return this.objects[ i ];
		}
	}
};

Timeline.prototype.getBaseCell = function() {
	for ( var i = 0; i < this.objects.length; i++ ) {
		if ( this.objects[ i ].isBase ) {
			return this.objects[ i ];
		}
	}
};

Timeline.prototype.setBaseCell = function( cell ) {
	for ( var i = 0; i < this.objects.length; i++ ) {
		this.objects[ i ].isBase = false;
	}
	cell.isBase = true;
};

Timeline.prototype.setSelected = function( ID, selected ) {
	for ( var i = 0; i < this.objects.length; i++ ) {
		if ( this.objects[ i ].ID == ID ) {
			this.objects[ i ].setSelectedState( selected );
		}
	}
};

Timeline.prototype.addObject = function( obj ) {
	if ( this.objects == null ) {
		this.objects = new Array();
	}
	this.dirty = true;
	this.objects.push( obj );
	if ( obj.div ) {
		this.setDivStyle( obj.div );
	}
	obj.parent = this;
};

Timeline.prototype.setOpenedStudies = function( studies ) {
	if ( studies === null ) {
		if ( this.openedStudies ) {
			studies = this.openedStudies;
		}
		else {
			return;
		}
	}
	else {
		this.openedStudies = studies;
	}
	studies = ";" + studies + ";";
	var res = "";
	for ( var i = 0; i < this.objects.length; i++ ) {
		var id = this.objects[ i ].ID;
		if ( id.startsWith( "LOCAL_" ) ) {
			id = id.substr( 6 );
			var isOpened = studies.indexOf( ";" + id + ";" ) > -1;
			var wasOpened = this.objects[ i ].type == "Opened" || this.objects[ i ].type == "Selected";
			if ( wasOpened != isOpened ) {
				this.objects[ i ].setType( isOpened ? "Opened" : "Study" );
			}
		}
		var p = id.indexOf( '_' );
		if ( p > -1 ) {
			id = id.substr( p + 1 );
		}
		res += ";" + id;
	}
	if ( res.length > 0 ) {
		res = res.substr( 1 );
	}
	if ( typeof this.tlbList != "undefined" ) {
		for ( var j = 0; j < this.tlbList.length; j++ ) {
			this.tlbList[ j ].refresh();
		}
	}
	return res;
};

Timeline.prototype.setRequestedStudies = function( studies ) {
	if ( studies === null ) {
		if ( this.requestedStudies ) {
			studies = this.requestedStudies;
		}
		else {
			return;
		}
	}
	else {
		this.requestedStudies = studies;
	}
	var basePos = null;
	var baseCell = null;
	studies = ";" + studies + ";";
	for ( var i = 0; i < this.objects.length; i++ ) {
		var id = this.objects[ i ].styiuid;
		var pos = studies.indexOf( ";" + id + ";" );
		if ( pos > -1 ) {
			// this is a requested study
			if ( basePos == null || pos < basePos ) {
				// this is a higher priority requested study than anything we
				// found so far
				basePos = pos;
				baseCell = this.objects[ i ];
			}
		}
		this.objects[ i ].setRequestedState( pos > -1 );
	}
	if ( baseCell && !baseCell.isBase ) {
		this.setBaseCell( baseCell );
		this.fetchRemote( -1 ); // should not start a separate refresh chain
	}
};

Timeline.prototype.checkWaiting = function() {
	if ( this.isLocal ) {
		if ( !this.waitingImg ) {

			this.waitingImg = $( "<img>", {
				"src" : "/art/img/waiting.gif",
				"style" : "position: absolute; bottom: 5px; left: " + ( ( $( this.rootElement ).width() - 100 ) / 2 ) + "px;"
			} ).get( 0 );
		}
		if ( this.waitingImg.parentNode !== this.rootElement ) {
			this.rootElement.appendChild( this.waitingImg );
		}
	}
	else {
		if ( this.waitingImg && this.waitingImg.parentNode === this.rootElement ) {
			this.rootElement.removeChild( this.waitingImg );
		}
	}
};

Timeline.prototype.setSelectedStudies = function( selected ) {
	selected = ";" + selected + ";";
	for ( var i = 0; i < this.objects.length; i++ ) {
		var id = this.objects[ i ].ID;
		if ( id.startsWith( "LOCAL_" ) ) {
			id = id.substr( 6 );
			var isSelected = selected.indexOf( ";" + id + ";" ) > -1;
			var wasSelected = this.objects[ i ].type == "Selected";
			if ( wasSelected != isSelected ) {
				this.objects[ i ].setType( isSelected ? "Selected" : "Opened" );
			}
		}
	}
	if ( typeof this.tlbList != "undefined" ) {
		for ( var j = 0; j < this.tlbList.length; j++ ) {
			this.tlbList[ j ].refresh();
		}
	}
};

Timeline.prototype.registerTimeLineButtons = function( tlb ) {
	if ( typeof this.tlbList == "undefined" ) {
		this.tlbList = new Array();
	}
	this.tlbList.push( tlb );
};

/**
 * 
 * this function queries the list of all cells (both local and remote) from the
 * server for refreshing the timline's cells
 * 
 * @param refreshVer
 *            number of the refresh chain (to avoid running multiple chains)
 * 
 */
Timeline.prototype.fetchRemote = function( refreshVer ) {
	this.isLocal = true;
	this.checkWaiting();
	var baseStudy = this.getBaseCell().styiuid;
	var rv = refreshVer;
	$.ajax( {
		url : "/cases/patientFolder.jsp?cRefresh=true&cStudy=" + baseStudy + "&tlID=" + this.root + "&timestamp=" + ( new Date() ).getTime(),
		dataType : 'json',
		context : this,
		success : function( res ) {
			if ( typeof res == 'string' ) {
				alert( "fetching remote Studies failed: " + res );
				return false;
			}

			var newCells = new Array();
			for ( var styiuid in res ) {
				var tcDesc = res[ styiuid ];
				var newTC = new IMTimelineCell( tcDesc );
				newCells.push( newTC );
			}
			var displayedStudy = this.getActualCell().styiuid;
			var newDispCellId = false;
			if ( this.dirtyCells && this.iframeID ) {
				// clear iframe cache for modified cells/studies
				for ( var i = 0; i < 2; i++ ) {
					for ( syui in this.dirtyCells[ i ] ) {
						for ( var j = 0; j < this.objects.length; j++ ) {
							if ( this.objects[ j ].styiuid === syui ) {
								unCacheImFramePage( this.iframeID, this.objects[ j ].ID );
								break;
							}
						}
					}
				}
			}
			this.clearObjects();
			for ( var i = 0; i < newCells.length; i++ ) {
				if ( newDispCellId === false || newCells[ i ].styiuid === displayedStudy ) {
					newDispCellId = newCells[ i ].ID;
				}
				this.addObject( newCells[ i ] );
			}
			this.setActual( newDispCellId );

			loadImFrame( this.iframeID, newDispCellId, false, true ); // force
			// load
			this.setOpenedStudies( null );
			this.setRequestedStudies( null );
			this.render();
			this.isLocal = false;
			this.checkWaiting();
		},
		complete : function() {
			this.scheduleRefresh( rv );
		}
	} );
};

/**
 * 
 * this function collects the timeline's current state map
 * 
 * @return the styiuid -> state map containing the local cells' states on the
 *         timeline
 * 
 */
Timeline.prototype.getStateMap = function() {
	var res = {};
	for ( var i = 0; i < this.objects.length; i++ ) {
		var cell = this.objects[ i ];
		if ( cell.type != "Remote" ) {
			res[ cell.styiuid ] = cell.StudyState;
		}
	}
	return res;
}

/**
 * 
 * this function checks whether two state objects are equal
 * 
 * @param stat1
 *            one state object
 * @param stat2
 *            another state object
 * 
 */
function statesEqual( stat1, stat2 ) {
	for ( var prp in stat1 ) {
		if ( !stat2.hasOwnProperty( prp ) ) {
			return false;
		}
		if ( stat1[ prp ].constructor === Array ) {
			if ( stat2[ prp ].constructor !== Array || stat1[ prp ].length != stat2[ prp ].length ) {
				return false;
			}
			for ( var i = 0; i < stat1[ prp ].length; i++ ) {
				if ( stat1[ prp ][ i ] !== stat2[ prp ][ i ] ) {
					return false;
				}
			}
		}
		else if ( stat1[ prp ] !== stat2[ prp ] ) {
			return false;
		}
	}
	for ( var prp in stat2 ) {
		if ( !stat1.hasOwnProperty( prp ) ) {
			return false;
		}
	}
	return true;
}

Timeline.prototype.m_refreshVer = 0;
Timeline.prototype.m_refreshTimer = 30000; // init at 30 seconds

/**
 * 
 * this function calculates and returns the next checkstate's timer
 * 
 */
Timeline.prototype.getRefreshTimer = function() {
	if ( this.m_refreshTimer != 30000 ) {
		this.m_refreshTimer = Math.min( 30000, Math.round( this.m_refreshTimer * 1.2 ) );
	}
	return this.m_refreshTimer;
}

/**
 * 
 * this function starts a new refresh chain
 * 
 */
Timeline.prototype.startRefreshChain = function() {
	this.m_refreshVer++;
	this.m_refreshTimer = 2000;
	this.scheduleRefresh( this.m_refreshVer, this.m_refreshTimer );
}

function console_log( msg ) {
	try {
		console.log( msg );
	}
	catch ( whoCares ) {
	}
}

/**
 * 
 * this function schedules a new checkstate call
 * 
 * @param refreshVer
 *            number of the refresh chain (to avoid running multiple chains)
 * 
 */
Timeline.prototype.scheduleRefresh = function( refreshVer, refreshTimer ) {
	if ( typeof refreshVer === "undefined" ) {
		refreshVer = this.m_refreshVer;
	}
	else if ( refreshVer != this.m_refreshVer ) {
		console_log( "stopping refresh chain #" + refreshVer );
		return;
	}
	if ( typeof refreshTimer === "undefined" ) {
		refreshTimer = this.getRefreshTimer();
	}
	console_log( "scheduling refresh chain #" + refreshVer + " @ " + refreshTimer + "ms" );
	// setTimeout( this.checkStates.bind( this, refreshVer ), refreshTimer ); //
	// retarded IE doesn't support bind
	var tl = this;
	setTimeout( function() {
		tl.checkStates( refreshVer )
	}, refreshTimer );
}

/**
 * 
 * this function compares the received local cell states and initiates a refresh
 * if needed
 * 
 * @param newStateMap
 *            the styiuid -> state map containing the server's current state
 * @param refreshVer
 *            number of the refresh chain (to avoid running multiple chains)
 * 
 */
Timeline.prototype.receiveStates = function( newStateMap, refreshVer ) {
	var oldStateMap = this.getStateMap();
	for ( var syui in newStateMap ) {
		if ( !oldStateMap[ syui ] ) {
			// new study
			continue;
		}
		if ( statesEqual( newStateMap[ syui ], oldStateMap[ syui ] ) ) {
			// matches
			delete newStateMap[ syui ];
		}
		delete oldStateMap[ syui ];
	}
	// oldStateMap now contains studies that have been deleted from the
	// worklist, newStateMap now contains new and modified studies

	// for now let's just refresh all cells if there's any change
	this.dirtyCells = [ oldStateMap, newStateMap ];
	for ( var prp in oldStateMap ) {
		this.fetchRemote( refreshVer );
		return;
	}
	for ( var prp in newStateMap ) {
		this.fetchRemote( refreshVer );
		return;
	}
	// otherwise schedule the next state check
	this.scheduleRefresh( refreshVer );
}

/**
 * 
 * this function starts a query to get the states of the local studies
 * 
 * @param refreshVer
 *            number of the refresh chain (to avoid running multiple chains)
 * 
 */
Timeline.prototype.checkStates = function( refreshVer ) {
	if ( refreshVer != this.m_refreshVer ) {
		console_log( "stopping refresh chain #" + refreshVer );
		return;
	}
	var baseStudy = this.getBaseCell().styiuid;
	$.ajax( {
		url : "/cases/timelineRefresh.jsp?r_no_refresh=true&cOp=status&cStudy=" + baseStudy + "&timestamp=" + ( new Date() ).getTime(),
		dataType : 'json',
		context : this,
		success : function( data ) {
			this.receiveStates( data, refreshVer );
		},
		error : function() {
			this.scheduleRefresh( refreshVer );
		}
	} );
};

function setOpenedStudies( studies ) {
	var res = "";
	for ( var i = 0; i < timeLines.length; i++ ) {
		var r = timeLines[ i ].setOpenedStudies( studies );
		if ( r.length > 0 ) {
			res += ";" + r;
		}
	}
	if ( res.length > 0 ) {
		res = res.substr( 1 );
	}
	return res;
};

function setNoteState( ID, hasNote ) {
	for ( var i = 0; i < timeLines.length; i++ ) {
		timeLines[ i ].setNoteState( ID, hasNote );
	}
};

function setSelectedStudies( selected ) {
	for ( var i = 0; i < timeLines.length; i++ ) {
		timeLines[ i ].setSelectedStudies( selected );
	}
};

function setRequestedStudies( selected ) {
	for ( var i = 0; i < timeLines.length; i++ ) {
		timeLines[ i ].setRequestedStudies( selected );
	}
};

function getActualTimeLineCellID() {
	// this function works correctly only if there is only one timeline
	return timeLines[ 0 ].getActualCell().ID;
};

function setActualTimeLineCellID( ID ) {
	// this function works correctly only if there is only one timeline
	timeLines[ 0 ].setActual( ID );
};

function refreshTimeline() {
	// this function works correctly only if there is only one timeline
	timeLines[ 0 ].fetchRemote();
	return true;
};

var imtlbMap = new Array();

function isNewViewer( minVer ) {
	if ( isWebkit ) {
		if ( !minVer ) {
			return true;
		}
		return window.app.PBViewerVersion > minVer;
	}
	try {
		var ver = window.external.getViewerVersion();
		if ( minVer ) {
			return ver > minVer;
		}
		return true;
	}
	catch ( e ) {
	}
	return false;
};

var ButtonTitles = new Array();

/**
 * 
 * this class creates/manages buttons optionally displayed along with timeline
 * containing studies buttons:
 * 
 * 0: Open study 1: retrieve study 2: Quick view 3: back to list
 * 
 * @param tlbID
 *            node containing the buttons and id of this IMTimeLineButtons
 *            instance as well
 * @param tlId
 *            ID of the timeline object
 */
function IMTimeLineButtons( tlbID, tlId, isIntegrated, isGetPbsServlet ) {
	this.id = tlbID;
	imtlbMap[ tlbID ] = this;
	this.isIntegrated = isIntegrated;
	this.disableOpen = isIntegrated && !isNewViewer();
	this.rootElement = document.getElementById( tlbID );
	this.rootElement.style.position = "relative";
	this.buttonWrappers = new Array();
	var button, labelDiv;
	var buttonStyles = isIntegrated ? [ "ViewerOpenButton", "RetrieveButton" ] : [ "OpenButton", "RetrieveButton", "QVButton", "BackButton" ];
	for ( var i = 0; i < buttonStyles.length; i++ ) {
		button = document.createElement( 'button' );
		button.className = "button";
		eval( "button.onclick = function( e ) { imtlbMap[ tlbID ].handleClick( " + i + " ); return false;};" );
		labelDiv = document.createElement( 'div' );
		labelDiv.className = buttonStyles[ i ];
		if ( ButtonTitles[ buttonStyles[ i ] ] ) {
			button.title = ButtonTitles[ buttonStyles[ i ] ];
			labelDiv.title = ButtonTitles[ buttonStyles[ i ] ];
		}
		button.appendChild( labelDiv );
		this.rootElement.appendChild( button );
		this.buttonWrappers.push( new ButtonWrapper( button ) );
	}
	if ( !tlId ) {
		tlId = timeLines[ 0 ].root;
	}
	this.timeLine = timeLineMap[ tlId ];
	this.timeLine.registerTimeLineButtons( this );

	this.handleClick = function( num ) {
		// if ( true ) {
		// alert( num );
		// return;
		// }
		var cell = this.timeLine.getActualCell();
		var styiuid = cell.styiuid;
		// 0 : quickview
		if ( num == 0 ) {
			if ( this.isIntegrated ) {
				callViewer( 'appendStudy', [ styiuid ] );
				// window.external.appendStudy( styiuid );
			}
			else {
				if ( ismobile ) {
					link = "eradv" + ( document.location.href.toLowerCase().startsWith( "https" ) ? "s" : "" ) + ":";
					var p0 = document.location.href.indexOf( '//' );
					var p = document.location.href.indexOf( '/', 9 );
					link += document.location.href.substring( p0, p );
					link += "/cases/describeStudy.jsp?sessionid=" + sid + "&cStudy=" + styiuid + "&cBack=" + urlEncode( document.location.href );
					document.location.href = link;
				}
				else {
					if ( isGetPbsServlet ) {
						document.location.href = "/GetPbs/" + ( new Date() ).getTime() + ".pbs?cGr=" + styiuid + "&Append=true";
					}
					else {
						document.location.href = "/cases/getpbs.jsp?cGr=" + styiuid + "&Append=true";
					}
				}
			}
			return;
		}
		if ( num == 1 ) {
			alert( IMgetContent( "/archive/get.jsp?cAjax=true&timestamp=" + ( new Date() ).getTime() + "&cDevStudy=" + cell.archiveDevice + "+" + styiuid
					+ "&cDesc_" + styiuid + "=" + escape( cell.displayStr ) ).text );
			// quicken refresh interval
			this.timeLine.startRefreshChain();
			return;
		}
		if ( num == 2 ) {
			document.location.href = "/cases/index.jsp?cQV=" + styiuid + "&cHidePrevNext=true";
			return;
		}
		if ( num == 3 ) {
			document.location.href = document.referrer;
		}
	};

	this.refresh = function() {
		var cell = this.timeLine.getActualCell();
		if ( cell ) {
			if ( cell.type ) {

				this.buttonWrappers[ 0 ].setEnabled( !this.disableOpen && cell.type == "Study" );
				this.buttonWrappers[ 1 ].setEnabled( cell.type == "Remote" );
				if ( !this.isIntegrated ) {
					this.buttonWrappers[ 2 ].setEnabled( cell.type != "Remote" );
					// back to list is always enabled
				}
			}
			else {
				alert( "Invalid cell object found: " + dumpObj( cell ) );
			}
		}
	};
};

/**
 * 
 * this class adds a toggable half-opaque layer which allow disabling/enabling
 * the original button
 * 
 * @param button
 */
function ButtonWrapper( button, baseStyleClass ) {
	this.button = button;
	this.baseStyleClass = baseStyleClass;

	this.setEnabled = function( enabled ) {
		var style = this.baseStyleClass || "button";
		if ( !enabled ) {
			style += " disabled";
		}
		this.button.disabled = !enabled;
		this.button.className = style;
	};
};
