#!/bin/sh
#
# Component: cases
#
# Purpose:
#     Handles STUDYDB released events
#
# Usage:
#    onStudydbReleased.sh -t <type> -e <event> -i <id>
#
#    -t type  : Object type to be released. It will always be "STUDYDB"
#    -e event : Event triggered runiing this script. It will always be "RELASED"
#    -i id    : Identifies the studydb object (=Study Instance UID)
#
# Description:
#     Will be called when a STUDYDB reference counter is released.
# $OBJECTTBL table entries will be removed for the study and reference counters of the objects
# will be decremented.
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#


. $HOME/etc/setenv.sh

# onStudydbReleased.sh can be called directly:
# run as a task instead
if [ `basename $0` = "onStudydbReleased.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi


LOGGER="/usr/bin/logger"
TAG="onStudydbReleased[$$]"

study=""
type=""
event=""
errMsg=""
INPUT_PARAMS=""

while getopts 'e:t:i:' OPT; do
	case $OPT in
	e) event="$OPTARG" ; INPUT_PARAMS="$INPUT_PARAMS -e \"$OPTARG\"" ;;
	t) type="$OPTARG" ; INPUT_PARAMS="$INPUT_PARAMS -t \"$OPTARG\"" ;;
	i) study="$OPTARG" ; INPUT_PARAMS="$INPUT_PARAMS -i \"$OPTARG\"" ;;
	*) errMsg="Invalid_Parameter:$OPT" ;;
	esac
done

[ -f "$CONF/taskd.rc" ] && . $CONF/taskd.rc

command="${CASES_HOME#$HOME/component/}.OnStudyDbReleased $INPUT_PARAMS"

run_()
{
	$TASKD_HOME/runjavacmd -c "0 $command" -p $JAVA_SERVER_PORT
	ERR=$?

	if [ ${ERR} -ne 0 ] ; then
		$TASKD_HOME/createtaskfileapp -p "RELEASE_REFERENCES" -t "class" -c "${command} -x"
		ERR=$?
	fi

	exit $ERR
}

cleanup_()
{
	$TASKD_HOME/runjavacmd -c "cleanup $command" -p $JAVA_SERVER_PORT
}

print_()
{
	$TASKD_HOME/runjavacmd -c "print $command" -p $JAVA_SERVER_PORT
}

