#!/bin/sh
#
# Purpose:
#     Performs a study operation on a remote dotcom server.
#
# Synopsis:
#    setstudyprp.sh [-u] -s <study> -o <op> -H <hostid>
#
# Arguments:
#     -s <study>  : Study Instance UID
#     -o <op>     : operation to perform on remote dotcom server
#     -H <hostid> : ID of the remote dotcom server to perform operation on
#     -u          : unlock study after successfull remote operation
#
# Description:
#
# OBSOLETE [7.0]
#
#   Performs on operation for a study on a remote dotcom server.
#
#   Such operations are (valid values for '-o op' argument):
#
#   | setowner         | set the child server to be the owner of the study
#   | setstaged        | add ourself to staged branch on the parent server
#   | removereference  | remove ourself from the staged branch on the parent server
#   | removesender     | remove ourself from the sender (owner) branch on the parent server
#   | nuke             | remove study from the dotcom
#   | passownership    | ask for study ownership from child server
#   | obsolete         | obsoletes a study both in dotcom and in the archive
#
#   When study ownership is passed between servers,
#   the study should be locked on both old and new owner server
#   After passing the ownership successfully, the lock can be
#   and should be removed on the old owner server.
#   Unlock (-u) parameter means that we are the old owner server
#   and if the ownership was passed successfully, release the unnecessary dead lock.
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#


. $HOME/etc/setenv.sh


# if setstudyprp.sh is called directly:
# run as a task instead
if [ `basename $0` = "setstudyprp.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi


LOGGER="/usr/bin/logger"
TAG="setstudyprp[$$]"

INPUT_PARAMS=""
study=""
oper=""
hostid=""
unlock="false"

while getopts 'us:o:H:' OPT; do
	case $OPT in
		u) unlock="true" ; INPUT_PARAMS="$INPUT_PARAMS -u" ;;
		s) study="$OPTARG" ; INPUT_PARAMS="$INPUT_PARAMS -s \"$OPTARG\"" ;;
		H) hostid="$OPTARG" ; INPUT_PARAMS="$INPUT_PARAMS -H \"$OPTARG\"" ;;
		o) oper="$OPTARG" ; INPUT_PARAMS="$INPUT_PARAMS -o \"$OPTARG\"" ;;
		*) exit 0 ;;
	esac
done

# include nettask.shi for checking if the device is temporary down
. $TASKD_HOME/nettask.shi

# override getPeer_() of nettask.shi
getPeer_()
{
	echo "$hostid"
}



run_() # override run_ method
{
	[ -z "$study" -o -z "$oper" -o -z "$hostid" ] && return 125
	$TASKD_HOME/runjavacmd -c "0 cases.SetStudyPrp $INPUT_PARAMS" -p "$JAVA_SERVER_PORT"
	ERR=$?

	exit $ERR
}

# override cleanup_
cleanup_()
{
	$TASKD_HOME/runjavacmd -c "cleanup cases.SetStudyPrp $INPUT_PARAMS" -p "$JAVA_SERVER_PORT"
}

# override print_
print_()
{
	$TASKD_HOME/runjavacmd -c "print cases.SetStudyPrp $INPUT_PARAMS" -p "$JAVA_SERVER_PORT"
}
 
