#!/bin/sh
#
# Component: iface
#
# Description:
#  Edits study
#
# Converts modfile into dcmodfile (dump2dcm) and calls $IFACE_HOME/bin/processedit
#
# Usage:
#     processedit.sh [-r] [-n] [-H | -S] [-o] -s <Study I UID> -f <modfile> [-c <config file] [-u <userid>] [-m <mmodfile>]
#
# For more information on usage see $IFACE_HOME/bin/processedit
#
# -m <mmodfile> : Contains createonly sequence from non merge order tags
#                 to be copied into study PbR file after merge
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

DontCreateLog=""
CreateNewOrder=""
StudyUid=""
EditStudy=""
UserID=""
ModFile=""
ConfigFile=""
RemoveFiles=""
MModFile=""
CoercionFile=""
EditMode=""

ERRMSG=""

while getopts 'rnoHSC:s:f:c:u:m:' OPT; do
	case $OPT in
	n) DontCreateLog="-n" ;;
	r) RemoveFiles="true" ;;
	o) CreateNewOrder="-o" ;;
	s) StudyUid="$OPTARG" ;;
	u) UserID="-u \"$OPTARG\"" ;;
	f) ModFile="$OPTARG" ;;
	c) ConfigFile="-c \"$OPTARG\"" ;;
	C) CoercionFile="-C \"$OPTARG\"" ;;
	m) MModFile="$OPTARG" ;;
	H) EditMode="-H" ;;
	S) EditMode="-S" ;;
	*) ERRMSG="Invalid_Parameter:$OPT" ;;
	esac
done

if [ -n "$ERRMSG" ] ; then
	echo "$ERRMSG" 1>&2
	exit 1
fi

[ -n "$StudyUid" ] && EditStudy="-s \"$StudyUid\""


Date=`date +"%Y%m%d%H%M%S"`

EditFileParam=""
EditFile=""
if [ -n "$ModFile" -o -f "$ModFile" ] ; then
	EditFile=$TMP/procedit.$Date.$$
	$DCVIEWER_HOME/dcmod -c -f $ModFile $EditFile
	[ $? -ne 0 ] && exit 1

	EditFileParam=" -f $EditFile"
fi

[ -n "$User" ] && User=" -u \"$User\""
[ -n "$MModFile" ] && MModFile=" -m \"$MModFile\""

PO_COMMAND="$SR_HOME/bin/processedit $EditFileParam $EditMode $DontCreateLog $CreateNewOrder $EditStudy $UserID $CoercionFile $MModFile $ConfigFile"

# echo $PO_COMMAND
eval $PO_COMMAND 2>/dev/null

Res=$?

[ -n "$EditFile" ] && rm $EditFile 2>/dev/null

if [ "true" = "$RemoveFiles" ] ; then
	rm "$ModFile" "$ConfigFile" "$MModFile" 2>/dev/null
fi

exit $Res
